@@ -21,8 +21,14 @@
 #endif
 
 int main(int argc, char **argv) {
+  QCoreApplication::setAttribute(Qt::AA_UseHighDpiPixmaps);
+#if QT_VERSION >= 0x050600
+  QCoreApplication::setAttribute(Qt::AA_EnableHighDpiScaling);
+#endif
+
   QApplication app(argc, argv);
-  app.setWindowIcon(QIcon(QString::fromUtf8(":/QtPDF/icons/logo.png")));
+  QIcon::setThemeName(QStringLiteral("tango-qtpdf"));
+  app.setWindowIcon(QIcon::fromTheme(QStringLiteral("QtPDF")));
   PDFViewer mainWin(QString::fromUtf8("pgfmanual.pdf"));
 
   mainWin.show();
