@@ -1,5 +1,5 @@
 /**
- * Copyright (C) 2011-2013  Charlie Sharpsteen, Stefan Löffler
+ * Copyright (C) 2013-2020  Charlie Sharpsteen, Stefan Löffler
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the Free
@@ -13,7 +13,26 @@
  */
 
 // NOTE: `PopplerQtBackend.h` is included via `PDFBackend.h`
-#include <PDFBackend.h>
+#include "PDFBackend.h"
+
+#include <QBitArray>
+
+#if defined(HAVE_POPPLER_XPDF_HEADERS) && defined(Q_OS_DARWIN)
+#include "poppler-config.h"
+#include "GlobalParams.h"
+#include <QCoreApplication>
+#include <QDir>
+#include <memory>
+#endif
+
+
+// Comparison operator for QSizeF needed to use QSizeF as keys in a QMap
+// NB: Must be in the global namespace
+inline bool operator<(const QSizeF & a, const QSizeF & b) {
+    qreal areaA = a.width() * a.height();
+    qreal areaB = b.width() * b.height();
+    return (areaA < areaB || (areaA == areaB && a.width() < b.width()));
+}
 
 namespace QtPDF {
 
@@ -29,7 +48,7 @@ PDFDestination toPDFDestination(const ::Poppler::Document * doc, const ::Poppler
 
   // Coordinates in LinkDestination are in the range of 0..1, which does not
   // comply with the pdf specs---so we have to convert them back
-  float w = 1., h = 1.;
+  qreal w = 1., h = 1.;
   if (doc) {
     ::Poppler::Page * p = doc->page(dest.pageNumber() - 1);
     if (p) {
@@ -79,7 +98,7 @@ PDFDestination toPDFDestination(const ::Poppler::Document * doc, const ::Poppler
   return retVal;
 }
 
-void convertAnnotation(Annotation::AbstractAnnotation * dest, const ::Poppler::Annotation * src, QWeakPointer<Backend::Page> thePage)
+void convertAnnotation(Annotation::AbstractAnnotation * dest, const ::Poppler::Annotation * src, const QWeakPointer<Backend::Page> & thePage)
 {
   QSharedPointer<Backend::Page> page(thePage.toStrongRef());
   if (!dest || !src || !page)
@@ -114,7 +133,7 @@ void convertAnnotation(Annotation::AbstractAnnotation * dest, const ::Poppler::A
     flags |= Annotation::AbstractAnnotation::Annotation_ToggleNoView;
 
   if (dest->isMarkup()) {
-    Annotation::Markup * annot = static_cast<Annotation::Markup*>(dest);
+    Annotation::Markup * annot = dynamic_cast<Annotation::Markup*>(dest);
     annot->setAuthor(src->author());
     annot->setCreationDate(src->creationDate());
   }
@@ -123,11 +142,9 @@ void convertAnnotation(Annotation::AbstractAnnotation * dest, const ::Poppler::A
 
 // Document Class
 // ==============
-Document::Document(QString fileName):
+Document::Document(const QString & fileName):
   Super(fileName),
-  _poppler_doc(::Poppler::Document::load(fileName)),
-  _poppler_docLock(new QMutex()),
-  _fontsLoaded(false)
+  _poppler_doc(::Poppler::Document::load(fileName))
 {
 #ifdef DEBUG
 //  qDebug() << "PopplerQt::Document::Document(" << fileName << ")";
@@ -173,6 +190,10 @@ void Document::parseDocument()
 {
   QWriteLocker docLocker(_docLock.data());
 
+  clearMetaData();
+  _meta_fileSize = QFileInfo(_fileName).size();
+  _numPages = -1;
+
   if (!_poppler_doc || _isLocked())
     return;
 
@@ -208,7 +229,6 @@ void Document::parseDocument()
   _poppler_doc->setRenderHint(::Poppler::Document::TextAntialiasing);
 
   // Load meta data
-  clearMetaData();
   QStringList metaKeys = _poppler_doc->infoKeys();
   if (metaKeys.contains(QString::fromUtf8("Title"))) {
     _meta_title = _poppler_doc->info(QString::fromUtf8("Title"));
@@ -243,6 +263,22 @@ void Document::parseDocument()
     metaKeys.removeAll(QString::fromUtf8("ModDate"));
   }
 
+  // Get the most often used page size
+  QMap<QSizeF, int> pageSizes;
+  for (int i = 0; i < _numPages; ++i) {
+    QSizeF ps = _poppler_doc->page(i)->pageSizeF();
+    if (pageSizes.contains(ps)) ++pageSizes[ps];
+    else pageSizes[ps] = 1;
+  }
+  int occurrences = -1;
+  _meta_pageSize = QSizeF();
+  Q_FOREACH(QSizeF ps, pageSizes.keys()) {
+      if (occurrences < pageSizes[ps]) {
+          _meta_pageSize = ps;
+          occurrences = pageSizes[ps];
+      }
+  }
+
   // Note: Poppler doesn't handle the meta data key "Trapped" correctly, as that
   // has a value of type `name` (/True, /False, or /Unknown) which doesn't get
   // converted to a string representation properly.
@@ -288,16 +324,6 @@ QWeakPointer<Backend::Page> Document::page(int at)
   return _pages[at].toWeakRef();
 }
 
-QWeakPointer<Backend::Page> Document::page(int at) const
-{
-  QReadLocker docLocker(_docLock.data());
-
-  if (at < 0 || at >= _numPages || at >= _pages.size())
-    return QWeakPointer<Backend::Page>();
-
-  return QWeakPointer<Backend::Page>(_pages[at]);
-}
-
 PDFDestination Document::resolveDestination(const PDFDestination & namedDestination) const
 {
   QReadLocker docLocker(_docLock.data());
@@ -307,13 +333,40 @@ PDFDestination Document::resolveDestination(const PDFDestination & namedDestinat
   if (namedDestination.isExplicit())
     return namedDestination;
 
-  // If the destination could not be resolved, return an invalid object
+  // If the destination could not be resolved (a nullptr or an invalid page
+  // number is returned), return an invalid object
   ::Poppler::LinkDestination * dest = _poppler_doc->linkDestination(namedDestination.destinationName());
-  if (!dest)
+  if (dest == nullptr || dest->pageNumber() < 1)
     return PDFDestination();
   return toPDFDestination(_poppler_doc.data(), *dest);
 }
 
+#if POPPLER_HAS_OUTLINE
+void Document::recursiveConvertToC(QList<PDFToCItem> & items, const QVector<Poppler::OutlineItem> & popplerItems) const
+{
+  for (const Poppler::OutlineItem & popplerItem : popplerItems) {
+    PDFToCItem newItem(popplerItem.name());
+    newItem.setOpen(popplerItem.isOpen());
+    // Note: color and flags are not supported by poppler
+
+    PDFGotoAction * action = nullptr;
+    if (popplerItem.destination())
+      action = new PDFGotoAction(toPDFDestination(_poppler_doc.data(), *(popplerItem.destination())));
+
+    if (action && !popplerItem.externalFileName().isEmpty()) {
+      // Open external links in new window by default (since poppler doesn't
+      // tell us what to do)
+      action->setOpenInNewWindow(true);
+      action->setRemote();
+      action->setFilename(popplerItem.externalFileName());
+    }
+    newItem.setAction(action);
+
+    recursiveConvertToC(newItem.children(), popplerItem.children());
+    items << newItem;
+  }
+}
+#else // POPPLER_HAS_OUTLINE
 void Document::recursiveConvertToC(QList<PDFToCItem> & items, QDomNode node) const
 {
   while (!node.isNull()) {
@@ -323,7 +376,7 @@ void Document::recursiveConvertToC(QList<PDFToCItem> & items, QDomNode node) con
     newItem.setOpen(attributes.namedItem(QString::fromUtf8("Open")).nodeValue() == QString::fromUtf8("true"));
     // Note: color and flags are not supported by poppler
 
-    PDFGotoAction * action = NULL;
+    PDFGotoAction * action = nullptr;
     QString val = attributes.namedItem(QString::fromUtf8("Destination")).nodeValue();
     if (!val.isEmpty())
       action = new PDFGotoAction(toPDFDestination(_poppler_doc.data(), ::Poppler::LinkDestination(val)));
@@ -350,6 +403,7 @@ void Document::recursiveConvertToC(QList<PDFToCItem> & items, QDomNode node) con
     node = node.nextSibling();
   }
 }
+#endif // POPPLER_HAS_OUTLINE
 
 PDFToC Document::toc() const
 {
@@ -359,11 +413,15 @@ PDFToC Document::toc() const
   if (!_poppler_doc || _isLocked())
     return retVal;
 
+#if POPPLER_HAS_OUTLINE
+  recursiveConvertToC(retVal, _poppler_doc->outline());
+#else // POPPLER_HAS_OUTLINE
   QDomDocument * toc = _poppler_doc->toc();
   if (!toc)
     return retVal;
   recursiveConvertToC(retVal, toc->firstChild());
   delete toc;
+#endif // POPPLER_HAS_OUTLINE
   return retVal;
 }
 
@@ -377,18 +435,14 @@ QList<PDFFontInfo> Document::fonts() const
   if (!_poppler_doc || _isLocked())
     return QList<PDFFontInfo>();
 
-  // Since ::Poppler::Document::fonts() is extremely slow, we need to cache the
-  // result. Since this function is declared const, we need to const_cast.
-  QList<PDFFontInfo> & fonts = const_cast<QList<PDFFontInfo>&>(_fonts);
-  bool & fontsLoaded = const_cast<bool&>(_fontsLoaded);
-  fontsLoaded = true;
+  _fontsLoaded = true;
 
   foreach(::Poppler::FontInfo popplerFontInfo, _poppler_doc->fonts()) {
     PDFFontInfo fi;
     if (popplerFontInfo.isEmbedded())
       fi.setSource(PDFFontInfo::Source_Embedded);
     else
-      fi.setFileName(popplerFontInfo.file());
+      fi.setFileName(QFileInfo(popplerFontInfo.file()));
     fi.setDescriptor(PDFFontDescriptor(popplerFontInfo.name()));
 
     switch (popplerFontInfo.type()) {
@@ -451,7 +505,7 @@ QList<PDFFontInfo> Document::fonts() const
       default:
         continue;
     }
-    fonts << fi;
+    _fonts << fi;
   }
   return _fonts;
 }
@@ -480,11 +534,9 @@ bool Document::unlock(const QString password)
 // Page Class
 // ==========
 Page::Page(Document *parent, int at, QSharedPointer<QReadWriteLock> docLock):
-  Super(parent, at, docLock),
-  _annotationsLoaded(false),
-  _linksLoaded(false)
+  Super(parent, at, docLock)
 {
-  _poppler_page = QSharedPointer< ::Poppler::Page >(static_cast<Document *>(_parent)->_poppler_doc->page(at));
+  _poppler_page = QSharedPointer< ::Poppler::Page >(dynamic_cast<Document *>(_parent)->_poppler_doc->page(at));
   loadTransitionData();
 }
 
@@ -500,11 +552,11 @@ QSizeF Page::pageSizeF() const
 {
   QReadLocker pageLocker(_pageLock);
 
-  Q_ASSERT(_poppler_page != NULL);
+  Q_ASSERT(_poppler_page != nullptr);
   return _poppler_page->pageSizeF();
 }
 
-QImage Page::renderToImage(double xres, double yres, QRect render_box, bool cache)
+QImage Page::renderToImage(double xres, double yres, QRect render_box, bool cache) const
 {
   QReadLocker docLocker(_docLock.data());
   QReadLocker pageLocker(_pageLock);
@@ -515,7 +567,7 @@ QImage Page::renderToImage(double xres, double yres, QRect render_box, bool cach
 
   {
     // Rendering pages is not thread safe.
-    QMutexLocker popplerDocLock(static_cast<Backend::PopplerQt::Document *>(_parent)->_poppler_docLock);
+    QMutexLocker popplerDocLock(dynamic_cast<Backend::PopplerQt::Document *>(_parent)->_poppler_docLock);
     if( render_box.isNull() ) {
       // A null QRect has a width and height of 0 --- we will tell Poppler to render the whole
       // page.
@@ -553,13 +605,13 @@ QList< QSharedPointer<Annotation::Link> > Page::loadLinks()
     return _links;
 
 
-  Q_ASSERT(_poppler_page != NULL);
+  Q_ASSERT(_poppler_page != nullptr);
   _linksLoaded = true;
   QList< ::Poppler::Link *> popplerLinks;
   QList< ::Poppler::Annotation *> popplerAnnots;
   {
     // Loading links is not thread safe.
-    QMutexLocker popplerDocLock(static_cast<Backend::PopplerQt::Document *>(_parent)->_poppler_docLock);
+    QMutexLocker popplerDocLock(dynamic_cast<Backend::PopplerQt::Document *>(_parent)->_poppler_docLock);
     popplerLinks = _poppler_page->links();
     popplerAnnots = _poppler_page->annotations();
   }
@@ -589,11 +641,11 @@ QList< QSharedPointer<Annotation::Link> > Page::loadLinks()
       if (!popplerAnnot || popplerAnnot->subType() != ::Poppler::Annotation::ALink || !denormalize.mapRect(popplerAnnot->boundary()).intersects(link->rect()))
         continue;
 
-      ::Poppler::LinkAnnotation * popplerLinkAnnot = static_cast< ::Poppler::LinkAnnotation *>(popplerAnnot);
+      ::Poppler::LinkAnnotation * popplerLinkAnnot = dynamic_cast< ::Poppler::LinkAnnotation *>(popplerAnnot);
       convertAnnotation(link.data(), popplerLinkAnnot, _parent->page(_n));
       // TODO: Does Poppler provide an easy interface to all quadPoints?
       // Note: ::Poppler::LinkAnnotation::HighlightMode is identical to PDFLinkAnnotation::HighlightingMode
-      link->setHighlightingMode((Annotation::Link::HighlightingMode)popplerLinkAnnot->linkHighlightMode());
+      link->setHighlightingMode(static_cast<Annotation::Link::HighlightingMode>(popplerLinkAnnot->linkHighlightMode()));
       break;
     }
 
@@ -602,8 +654,8 @@ QList< QSharedPointer<Annotation::Link> > Page::loadLinks()
     switch (popplerLink->linkType()) {
       case ::Poppler::Link::Goto:
         {
-          ::Poppler::LinkGoto * popplerGoto = static_cast< ::Poppler::LinkGoto *>(popplerLink);
-          PDFGotoAction * action = new PDFGotoAction(toPDFDestination(static_cast<Document *>(_parent)->_poppler_doc.data(), popplerGoto->destination()));
+          ::Poppler::LinkGoto * popplerGoto = dynamic_cast< ::Poppler::LinkGoto *>(popplerLink);
+          PDFGotoAction * action = new PDFGotoAction(toPDFDestination(dynamic_cast<Document *>(_parent)->_poppler_doc.data(), popplerGoto->destination()));
           if (popplerGoto->isExternal()) {
             // TODO: Verify that ::Poppler::LinkGoto only refers to pdf files
             // (for other file types we would need PDFLaunchAction)
@@ -616,15 +668,15 @@ QList< QSharedPointer<Annotation::Link> > Page::loadLinks()
         break;
       case ::Poppler::Link::Execute:
         {
-          ::Poppler::LinkExecute * popplerExecute = static_cast< ::Poppler::LinkExecute *>(popplerLink);
+          ::Poppler::LinkExecute * popplerExecute = dynamic_cast< ::Poppler::LinkExecute *>(popplerLink);
           if (popplerExecute->parameters().isEmpty())
             link->setActionOnActivation(new PDFLaunchAction(popplerExecute->fileName()));
           else
-            link->setActionOnActivation(new PDFLaunchAction(QString::fromUtf8("%1 %2").arg(popplerExecute->fileName()).arg(popplerExecute->parameters())));
+            link->setActionOnActivation(new PDFLaunchAction(QString::fromUtf8("%1 %2").arg(popplerExecute->fileName(), popplerExecute->parameters())));
         }
         break;
       case ::Poppler::Link::Browse:
-        link->setActionOnActivation(new PDFURIAction(static_cast< ::Poppler::LinkBrowse*>(popplerLink)->url()));
+        link->setActionOnActivation(new PDFURIAction(dynamic_cast< ::Poppler::LinkBrowse*>(popplerLink)->url()));
         break;
       /*
       case ::Poppler::Link::Action:
@@ -665,7 +717,7 @@ QList< QSharedPointer<Annotation::AbstractAnnotation> > Page::loadAnnotations()
   QList< ::Poppler::Annotation *> popplerAnnots;
   {
     // Loading annotations is not thread safe.
-    QMutexLocker popplerDocLock(static_cast<Document *>(_parent)->_poppler_docLock);
+    QMutexLocker popplerDocLock(dynamic_cast<Document *>(_parent)->_poppler_docLock);
     popplerAnnots = _poppler_page->annotations();
   }
 
@@ -700,7 +752,7 @@ QList< QSharedPointer<Annotation::AbstractAnnotation> > Page::loadAnnotations()
       }
       case ::Poppler::Annotation::AHighlight:
       {
-        ::Poppler::HighlightAnnotation * popplerHighlight = static_cast< ::Poppler::HighlightAnnotation*>(popplerAnnot);
+        ::Poppler::HighlightAnnotation * popplerHighlight = dynamic_cast< ::Poppler::HighlightAnnotation*>(popplerAnnot);
         switch (popplerHighlight->highlightType()) {
           case ::Poppler::HighlightAnnotation::Highlight:
           {
@@ -749,36 +801,37 @@ QList< QSharedPointer<Annotation::AbstractAnnotation> > Page::loadAnnotations()
   return _annotations;
 }
 
-QList<SearchResult> Page::search(QString searchText, SearchFlags flags)
+QList<SearchResult> Page::search(const QString & searchText, const SearchFlags & flags)
 {
   QList<SearchResult> results;
   SearchResult result;
-  double left, right, top, bottom;
-  ::Poppler::Page::SearchDirection searchDir = (flags & Search_Backwards ? ::Poppler::Page::PreviousResult : ::Poppler::Page::NextResult);
-  ::Poppler::Page::SearchMode searchMode = (flags & Search_CaseInsensitive ? ::Poppler::Page::CaseInsensitive : ::Poppler::Page::CaseSensitive);
+  double left{0}, right{0}, top{0}, bottom{0};
+  ::Poppler::Page::SearchDirection searchDir = (flags.testFlag(Search_Backwards) ? ::Poppler::Page::PreviousResult : ::Poppler::Page::NextResult);
+#if POPPLER_HAS_SEARCH_FLAGS
+  ::Poppler::Page::SearchFlags searchFlags = (flags.testFlag(Search_CaseInsensitive) ? ::Poppler::Page::IgnoreCase : ::Poppler::Page::SearchFlags());
+#else
+  ::Poppler::Page::SearchMode searchFlags = (flags.testFlag(Search_CaseInsensitive) ? ::Poppler::Page::CaseInsensitive : ::Poppler::Page::CaseSensitive);
+#endif
 
   QReadLocker docLocker(_docLock.data());
   QReadLocker pageLocker(_pageLock);
   if (!_parent)
     return results;
 
-  result.pageNum = _n;
+  result.pageNum = static_cast<unsigned int>(_n);
 
-  QMutexLocker popplerDocLock(static_cast<Document *>(_parent)->_poppler_docLock);
+  QMutexLocker popplerDocLock(dynamic_cast<Document *>(_parent)->_poppler_docLock);
 
   if (flags & Search_Backwards) {
     left = right = pageSizeF().width();
     top = bottom = pageSizeF().height();
   }
-  else {
-    left = top = right = bottom = 0;
-  }
 
   // The Poppler search function that takes a QRectF has been marked as
   // depreciated---something to do with float <-> double conversion causing
   // infinite loops on some architectures. So, we explicitly use doubles and
   // avoid the depreciated function.
-  while ( _poppler_page->search(searchText, left, top, right, bottom, searchDir, searchMode) ) {
+  while ( _poppler_page->search(searchText, left, top, right, bottom, searchDir, searchFlags) ) {
     result.bbox = QRectF(qreal(left), qreal(top), qAbs(qreal(right) - qreal(left)), qAbs(qreal(bottom) - qreal(top)));
     results << result;
   }
@@ -856,7 +909,11 @@ void Page::loadTransitionData()
       break;
     }
     if (_transition) {
+#if POPPLER_HAS_DURATION_REAL
+      _transition->setDuration(poppler_trans->durationReal());
+#else
       _transition->setDuration(poppler_trans->duration());
+#endif
       switch (poppler_trans->direction()) {
       case ::Poppler::PageTransition::Inward:
       default:
@@ -872,7 +929,7 @@ void Page::loadTransitionData()
 QList< Backend::Page::Box > Page::boxes()
 {
   QReadLocker pageLocker(_pageLock);
-  Q_ASSERT(_poppler_page != NULL);
+  Q_ASSERT(_poppler_page != nullptr);
   QList< Backend::Page::Box > retVal;
 
   foreach (::Poppler::TextBox * popplerTextBox, _poppler_page->textList()) {
@@ -890,10 +947,10 @@ QList< Backend::Page::Box > Page::boxes()
   return retVal;
 }
 
-QString Page::selectedText(const QList<QPolygonF> & selection, QMap<int, QRectF> * wordBoxes /* = NULL */, QMap<int, QRectF> * charBoxes /* = NULL */, const bool onlyFullyEnclosed /* = false */)
+QString Page::selectedText(const QList<QPolygonF> & selection, QMap<int, QRectF> * wordBoxes /* = nullptr */, QMap<int, QRectF> * charBoxes /* = nullptr */, const bool onlyFullyEnclosed /* = false */)
 {
   QReadLocker pageLocker(_pageLock);
-  Q_ASSERT(_poppler_page != NULL);
+  Q_ASSERT(_poppler_page != nullptr);
   // Using the bounding rects of the selection polygons is almost
   // certainly wrong! However, poppler-qt4 doesn't offer any alternative AFAICS
   // (except for positioning each char in the string manually).
@@ -901,90 +958,99 @@ QString Page::selectedText(const QList<QPolygonF> & selection, QMap<int, QRectF>
   // list of words. Hence, by iterating over them, we get a list of words with
   // no whitespace inbetween
   QString retVal;
+  bool insertSpace = false;
 
-	// Get a list of all boxes
-	QList<Poppler::TextBox*> poppler_boxes = _poppler_page->textList();
-	Poppler::TextBox * lastPopplerBox = NULL;
+  // Get a list of all boxes
+  QList<Poppler::TextBox*> poppler_boxes = _poppler_page->textList();
+  Poppler::TextBox * lastPopplerBox = nullptr;
 
-	// Filter boxes by selection
-	foreach (Poppler::TextBox * poppler_box, poppler_boxes) {
-		if (!poppler_box)
-			continue;
-		bool include = false;
-		bool includeEntirely = false;
-		foreach (const QPolygonF & p, selection) {
-			if (!p.intersected(poppler_box->boundingBox()).empty()) {
-				include = true;
-				includeEntirely = QPolygonF(poppler_box->boundingBox()).subtracted(p).empty();
-				break;
-			}
-		}
-		if (!include)
-			continue;
-		// If we get here, we found a box in the selection, so we append its text
+  // Filter boxes by selection
+  foreach (Poppler::TextBox * poppler_box, poppler_boxes) {
+    if (!poppler_box)
+      continue;
 
-		// Guess ends of line: if the new box is entirely below the old box, we
-		// assume it's a new line. This should work reasonably well for normal text
-		// (including RTL text), but may fail in some less common cases (e.g.,
-		// subscripts after superscripts, formulas, etc.).
-		if (lastPopplerBox && lastPopplerBox->boundingBox().bottom() < poppler_box->boundingBox().top()) {
-			retVal += QString::fromLatin1("\n");
+    // Determine which characters to include (if any)
+    QBitArray include(poppler_box->text().length());
+    for (int i = 0; i < poppler_box->text().length(); ++i) {
+      QPolygonF remainder(poppler_box->charBoundingBox(i));
+      foreach (const QPolygonF & p, selection) {
+        // Include characters if they are entirely inside the selection area or
+        // onlyFullyEnclosed == false; using "intersection only" can cause
+        // problems for overlapping char boxes (if the selection is made of
+        // entire char boxes, it would return characters that are not actually
+        // inside the selection but are just "edge cases") but is necessary if
+        // the selection comes from external sources, such as SyncTeX
+        if (p.intersected(poppler_box->charBoundingBox(i)).empty())
+          continue;
+        if (!onlyFullyEnclosed) {
+          include.setBit(i);
+          break;
+        }
+        remainder = remainder.subtracted(p);
+        if (remainder.empty()) {
+          include.setBit(i);
+          break;
+        }
+      }
+    }
+    if (include.count(true) == 0) continue;
 
-			if (wordBoxes)
-				(*wordBoxes)[wordBoxes->count()] = lastPopplerBox->boundingBox();
-			if (charBoxes)
-				(*charBoxes)[charBoxes->count()] = lastPopplerBox->boundingBox();
-		}
+    // If we get here, we found a box that is at least partially selected, so we
+    // append the appropriate text
 
-		bool appendSpace = false;
-		if (includeEntirely) {
-			retVal += poppler_box->text();
-			if (wordBoxes) {
-				for (int i = 0; i < poppler_box->text().length(); ++i)
-					(*wordBoxes)[wordBoxes->count()] = poppler_box->boundingBox();
-			}
-			if (charBoxes) {
-				for (int i = 0; i < poppler_box->text().length(); ++i)
-					(*charBoxes)[charBoxes->count()] = poppler_box->charBoundingBox(i);
-			}
-			appendSpace = poppler_box->hasSpaceAfter();
-		}
-		else {
-			for (int i = 0; i < poppler_box->text().length(); ++i) {
-				foreach (const QPolygonF & p, selection) {
-					// Append text for char boxes if they are entirely inside the
-					// selection area or onlyFullyEnclosed == false; using "intersection
-					// only" can cause problems for overlapping char boxes (if selection
-					// is made of entire char boxes, it would return characters that are
-					// not actually inside the selection but are just "edge cases") but is
-					// necessary if selection comes from external sources, such as SyncTeX
-					if (p.intersected(poppler_box->charBoundingBox(i)).empty())
-						continue;
-					if (!onlyFullyEnclosed || QPolygonF(poppler_box->charBoundingBox(i)).subtracted(p).empty()) {
-						retVal += poppler_box->text()[i];
+    // Guess ends of lines: if the new box is mostly below the old box (with the
+    // overlap being less than 20% of the height of the larger box), we assume
+    // it's a new line. This should work reasonably well for normal text
+    // (including RTL text), but may fail in some less common cases (e.g.,
+    // subscripts after superscripts, formulas, etc.).
+    if (lastPopplerBox && lastPopplerBox->boundingBox().bottom() - poppler_box->boundingBox().top() < 0.2 * qMax(lastPopplerBox->boundingBox().height(), poppler_box->boundingBox().height())) {
+      retVal += QString::fromLatin1("\n");
 
-						if (wordBoxes)
-							(*wordBoxes)[wordBoxes->count()] = poppler_box->boundingBox();
-						if (charBoxes)
-							(*charBoxes)[charBoxes->count()] = poppler_box->charBoundingBox(i);
+      if (wordBoxes)
+        (*wordBoxes)[wordBoxes->count()] = lastPopplerBox->boundingBox();
+      if (charBoxes)
+        (*charBoxes)[charBoxes->count()] = lastPopplerBox->boundingBox();
+      // If we queued a space to be inserted, ignore that as we inserted a
+      // newline instead anyway
+      insertSpace = false;
+    }
 
-						if (i == poppler_box->text().length() - 1)
-							appendSpace = poppler_box->hasSpaceAfter();
-						break;
-					}
-				}
-			}
-		}
-		if (appendSpace) {
-			retVal += QString::fromLatin1(" ");
+    if (insertSpace && lastPopplerBox) {
+      retVal += QString::fromLatin1(" ");
 
-			if (wordBoxes)
-				(*wordBoxes)[wordBoxes->count()] = poppler_box->boundingBox();
-			if (charBoxes)
-				(*charBoxes)[charBoxes->count()] = poppler_box->boundingBox();
-		}
-		lastPopplerBox = poppler_box;
-	}
+      // As word and char Boxes, insert those of the lastPopplerBox since that
+      // was the one causing insertSpace to be true
+      if (wordBoxes)
+        (*wordBoxes)[wordBoxes->count()] = lastPopplerBox->boundingBox();
+      if (charBoxes)
+        (*charBoxes)[charBoxes->count()] = lastPopplerBox->boundingBox();
+    }
+
+    // Default to not inserting a space after this word
+    insertSpace = false;
+
+    // Insert the actual characters
+    for (int i = 0; i < poppler_box->text().length(); ++i) {
+      if (!include.testBit(i)) continue;
+
+      retVal += poppler_box->text()[i];
+
+      if (wordBoxes)
+        (*wordBoxes)[wordBoxes->count()] = poppler_box->boundingBox();
+      if (charBoxes)
+        (*charBoxes)[charBoxes->count()] = poppler_box->charBoundingBox(i);
+
+      // If we reached the end of the word, possibly queue a space to be
+      // inserted. By queuing this until the next word is processed, we ensure
+      // that spaces are not inserted at the end of the string or before
+      // newlines
+      if (i == poppler_box->text().length() - 1)
+        insertSpace = poppler_box->hasSpaceAfter();
+    }
+    // Remember the last processed box (required for detecting newlines and
+    // inserting spaces)
+    lastPopplerBox = poppler_box;
+  }
 
   return retVal;
 }
@@ -993,6 +1059,35 @@ QString Page::selectedText(const QList<QPolygonF> & selection, QMap<int, QRectF>
 
 } // namespace Backend
 
+QSharedPointer<Backend::Document> PopplerQtBackend::newDocument(const QString & fileName) {
+#if defined(HAVE_POPPLER_XPDF_HEADERS) && defined(Q_OS_DARWIN)
+  static bool globalParamsInitialized = false;
+  if (!globalParamsInitialized) {
+    globalParamsInitialized = true;
+    #if defined(POPPLER_HAS_GLOBALPARAMSINITER)
+      QDir dataDir{QCoreApplication::applicationDirPath()};
+      if (dataDir.cd(QStringLiteral("../share/poppler"))) {
+        GlobalParamsIniter::setCustomDataDir(qPrintable(dataDir.path()));
+      }
+    #else // defined(POPPLER_HAS_GLOBALPARAMSINITER)
+      // for Mac, support "local" poppler-data directory
+      // (requires patched poppler-qt lib to be effective,
+      // otherwise the GlobalParams gets overwritten when a
+      // document is opened)
+      QDir dataDir{QCoreApplication::applicationDirPath()};
+      if (dataDir.cd(QStringLiteral("../share/poppler"))) {
+        #if defined(POPPLER_GLOBALPARAMS_IS_UNIQUE)
+          globalParams = std::move(std::unique_ptr<GlobalParams>(new GlobalParams(qPrintable(dataDir.path()))));
+        #else
+          globalParams = new GlobalParams(qPrintable(dataDir.path()));
+        #endif
+      }
+    #endif // defined(POPPLER_HAS_GLOBALPARAMSINITER)
+  }
+#endif // defined(HAVE_POPPLER_XPDF_HEADERS) && defined(Q_OS_DARWIN)
+  return QSharedPointer<Backend::Document>(new Backend::PopplerQt::Document(fileName));
+}
+
 } // namespace QtPDF
 
 // vim: set sw=2 ts=2 et
