@@ -1,5 +1,5 @@
 /**
- * Copyright (C) 2013-2018  Charlie Sharpsteen, Stefan Löffler
+ * Copyright (C) 2013-2020  Charlie Sharpsteen, Stefan Löffler
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the Free
@@ -22,10 +22,11 @@
 #define PopplerBackend_H
 
 #include "PDFBackend.h"
-#if QT_VERSION < 0x050000
-  #include <poppler-qt4.h>
+
+#if QT_VERSION >= QT_VERSION_CHECK(6, 0, 0)
+#include <poppler-qt6.h>
 #else
-  #include <poppler-qt5.h>
+#include <poppler-qt5.h>
 #endif
 
 namespace QtPDF {
@@ -44,36 +45,41 @@ class Document: public Backend::Document
 
   QSharedPointer< ::Poppler::Document > _poppler_doc;
 
+#if POPPLER_HAS_OUTLINE
+  void recursiveConvertToC(QList<PDFToCItem> & items, const QVector<Poppler::OutlineItem> & popplerItems) const;
+#else
   void recursiveConvertToC(QList<PDFToCItem> & items, QDomNode node) const;
+#endif
 
 protected:
   // Poppler is not threadsafe, so some operations need to be serialized with a
   // mutex.
-  QMutex * _poppler_docLock;
-  QList<PDFFontInfo> _fonts;
-  bool _fontsLoaded;
+  QMutex * _poppler_docLock{new QMutex};
+  // Since ::Poppler::Document::fonts() is extremely slow, we need to cache the
+  // result.
+  mutable QList<PDFFontInfo> _fonts;
+  mutable bool _fontsLoaded{false};
 
   // The following two methods are not thread-safe because they don't acquire a
   // read lock. This is to enable methods that have a write lock to use them.
-  bool _isValid() const { return (_poppler_doc != NULL); }
+  bool _isValid() const { return (_poppler_doc != nullptr); }
   bool _isLocked() const { return (_poppler_doc ? _poppler_doc->isLocked() : false); }
 
 public:
-  Document(QString fileName);
-  ~Document();
+  Document(const QString & fileName);
+  ~Document() override;
 
-  bool isValid() const { QReadLocker docLocker(_docLock.data()); return _isValid(); }
-  bool isLocked() const { QReadLocker docLocker(_docLock.data()); return _isLocked(); }
+  bool isValid() const override { QReadLocker docLocker(_docLock.data()); return _isValid(); }
+  bool isLocked() const override { QReadLocker docLocker(_docLock.data()); return _isLocked(); }
 
-  void reload();
-  bool unlock(const QString password);
+  void reload() override;
+  bool unlock(const QString password) override;
 
-  QWeakPointer<Backend::Page> page(int at);
-  QWeakPointer<Backend::Page> page(int at) const;
-  PDFDestination resolveDestination(const PDFDestination & namedDestination) const;
+  QWeakPointer<Backend::Page> page(int at) override;
+  PDFDestination resolveDestination(const PDFDestination & namedDestination) const override;
 
-  PDFToC toc() const;
-  QList<PDFFontInfo> fonts() const;
+  PDFToC toc() const override;
+  QList<PDFFontInfo> fonts() const override;
 
 private:
   void parseDocument();
@@ -88,8 +94,8 @@ class Page: public Backend::Page
   QSharedPointer< ::Poppler::Page > _poppler_page;
   QList< QSharedPointer<Annotation::AbstractAnnotation> > _annotations;
   QList< QSharedPointer<Annotation::Link> > _links;
-  bool _annotationsLoaded;
-  bool _linksLoaded;
+  bool _annotationsLoaded{false};
+  bool _linksLoaded{false};
 
   void loadTransitionData();
 
@@ -97,18 +103,18 @@ protected:
   Page(Document *parent, int at, QSharedPointer<QReadWriteLock> docLock);
 
 public:
-  ~Page();
+  ~Page() override;
 
-  QSizeF pageSizeF() const;
+  QSizeF pageSizeF() const override;
 
-  QImage renderToImage(double xres, double yres, QRect render_box = QRect(), bool cache = false) const;
+  QImage renderToImage(double xres, double yres, QRect render_box = QRect(), bool cache = false) const override;
 
-  QList< QSharedPointer<Annotation::Link> > loadLinks();
-  QList< QSharedPointer<Annotation::AbstractAnnotation> > loadAnnotations();
-  QList< Backend::Page::Box > boxes();
-  QString selectedText(const QList<QPolygonF> & selection, QMap<int, QRectF> * wordBoxes = NULL, QMap<int, QRectF> * charBoxes = NULL, const bool onlyFullyEnclosed = false);
+  QList< QSharedPointer<Annotation::Link> > loadLinks() override;
+  QList< QSharedPointer<Annotation::AbstractAnnotation> > loadAnnotations() override;
+  QList< Backend::Page::Box > boxes() override;
+  QString selectedText(const QList<QPolygonF> & selection, QMap<int, QRectF> * wordBoxes = nullptr, QMap<int, QRectF> * charBoxes = nullptr, const bool onlyFullyEnclosed = false) override;
 
-  QList<Backend::SearchResult> search(QString searchText, SearchFlags flags);
+  QList<Backend::SearchResult> search(const QString & searchText, const SearchFlags & flags) override;
 };
 
 } // namespace PopplerQt
@@ -120,15 +126,10 @@ class PopplerQtBackend : public BackendInterface
   Q_OBJECT
   Q_INTERFACES(QtPDF::BackendInterface)
 public:
-  PopplerQtBackend() { }
-  virtual ~PopplerQtBackend() { }
+  QSharedPointer<Backend::Document> newDocument(const QString & fileName) override;
 
-  virtual QSharedPointer<Backend::Document> newDocument(const QString & fileName) {
-    return QSharedPointer<Backend::Document>(new Backend::PopplerQt::Document(fileName));
-  }
-
-  virtual QString name() const { return QString::fromLatin1("poppler-qt"); }
-  virtual bool canHandleFile(const QString & fileName) { return QFileInfo(fileName).suffix() == QString::fromLatin1("pdf"); }
+  QString name() const override { return QString::fromLatin1("poppler-qt"); }
+  bool canHandleFile(const QString & fileName) override { return QFileInfo(fileName).suffix() == QString::fromLatin1("pdf"); }
 };
 
 } // namespace QtPDF
