@@ -1,6 +1,6 @@
 /*
 	This is part of TeXworks, an environment for working with TeX documents
-	Copyright (C) 2007-2018  Jonathan Kew, Stefan Löffler, Charlie Sharpsteen
+	Copyright (C) 2007-2020  Jonathan Kew, Stefan Löffler, Charlie Sharpsteen
 
 	This program is free software; you can redistribute it and/or modify
 	it under the terms of the GNU General Public License as published by
@@ -20,42 +20,42 @@
 */
 
 #include "CompletingEdit.h"
-#include "TWUtils.h"
-#include "TWApp.h"
 
-#include <QCompleter>
-#include <QKeyEvent>
+#include "DefaultPrefs.h"
+#include "Settings.h"
+#include "TWApp.h"
+#include "TWUtils.h"
+#include "TeXHighlighter.h"
+#include "document/TeXDocument.h"
+#include "utils/ResourcesLibrary.h"
+
 #include <QAbstractItemView>
+#include <QAbstractTextDocumentLayout>
 #include <QApplication>
+#include <QClipboard>
+#include <QCompleter>
+#include <QDir>
+#include <QFile>
+#include <QFileInfo>
+#include <QKeyEvent>
+#include <QMenu>
 #include <QModelIndex>
+#include <QPainter>
+#include <QScrollBar>
+#include <QSignalMapper>
 #include <QStandardItem>
 #include <QStandardItemModel>
-#include <QTextCursor>
-#include <QFileInfo>
-#include <QFile>
-#include <QDir>
-#include <QMenu>
-#include <QTextStream>
-#include <QTextCodec>
-#include <QAbstractTextDocumentLayout>
-#include <QSignalMapper>
-#include <QTextDocument>
 #include <QTextBlock>
-#include <QScrollBar>
+#include <QTextCodec>
+#include <QTextCursor>
+#include <QTextStream>
 #include <QTimer>
-#include <QPainter>
-#include <QClipboard>
 
-CompletingEdit::CompletingEdit(QWidget *parent)
-	: QTextEdit(parent),
-	  clickCount(0),
-	  wheelDelta(0),
-	  autoIndentMode(-1), prefixLength(0),
-	  smartQuotesMode(-1),
-	  c(NULL), cmpCursor(QTextCursor()),
-	  pHunspell(NULL), spellingCodec(NULL)
+CompletingEdit::CompletingEdit(QWidget *parent /* = nullptr */)
+	: QTextEdit(parent)
 {
-	if (sharedCompleter == NULL) { // initialize shared (static) members
+	Tw::Settings settings;
+	if (!sharedCompleter) { // initialize shared (static) members
 		sharedCompleter = new QCompleter(qApp);
 		sharedCompleter->setCompletionMode(QCompleter::InlineCompletion);
 		sharedCompleter->setCaseSensitivity(Qt::CaseInsensitive);
@@ -65,28 +65,48 @@ CompletingEdit::CompletingEdit(QWidget *parent)
 		braceMatchingFormat = new QTextCharFormat;
 		currentLineFormat = new QTextCharFormat;
 
-		QSETTINGS_OBJECT(settings);
 		highlightCurrentLine = settings.value(QString::fromLatin1("highlightCurrentLine"), true).toBool();
 		autocompleteEnabled = settings.value(QString::fromLatin1("autocompleteEnabled"), true).toBool();
 	}
-		
+	setCursorWidth(settings.value(QStringLiteral("cursorWidth"), kDefault_CursorWidth).toInt());
+
 	loadIndentModes();
 	loadSmartQuotesModes();
-	
-	connect(this, SIGNAL(cursorPositionChanged()), this, SLOT(cursorPositionChangedSlot()));
-	connect(this, SIGNAL(selectionChanged()), this, SLOT(cursorPositionChangedSlot()));
 
-	lineNumberArea = new LineNumberArea(this);
-	
-	connect(document(), SIGNAL(blockCountChanged(int)), this, SLOT(updateLineNumberAreaWidth(int)));
-	connect(this, SIGNAL(updateRequest(const QRect&, int)), this, SLOT(updateLineNumberArea(const QRect&, int)));
-	connect(this, SIGNAL(textChanged()), lineNumberArea, SLOT(update()));
+	connect(this, &CompletingEdit::cursorPositionChanged, this, &CompletingEdit::cursorPositionChangedSlot);
+	connect(this, &CompletingEdit::selectionChanged, this, &CompletingEdit::cursorPositionChangedSlot);
+
+	lineNumberArea = new Tw::UI::LineNumberWidget(this);
+
+	// Invoke our setDocument() method to properly set up document-specific
+	// connections
+	setDocument(document());
+	connect(this, &CompletingEdit::updateRequest, this, &CompletingEdit::updateLineNumberArea);
+	connect(this, &CompletingEdit::textChanged, lineNumberArea, static_cast<void (Tw::UI::LineNumberWidget::*)()>(&Tw::UI::LineNumberWidget::update));
+
+	connect(TWApp::instance(), &TWApp::highlightLineOptionChanged, this, &CompletingEdit::resetExtraSelections);
+
+	setupUi(this);
+	connect(actionJump_To_PDF, &QAction::triggered, this, [=]() { this->jumpToPdf(); });
+	// As these actions are not used in menus/toolbars, we need to manually add
+	// them to the widget for TWUtils::installCustomShortcuts to work
+	insertActions(nullptr, {actionNext_Completion, actionPrevious_Completion, actionNext_Completion_Placeholder, actionPrevious_Completion_Placeholder, actionJump_To_PDF});
+
+#ifdef Q_OS_DARWIN
+	// Backwards compatibility
+	// Ctrl+Tab is mapped to Command+Tab on the Mac, which is the standard key
+	// sequence for switching applications. Hence that combination is changed to
+	// Alt+Tab on the Mac
+	if (actionNext_Completion_Placeholder->shortcut() == QKeySequence(Qt::CTRL + Qt::Key_Tab))
+		actionNext_Completion_Placeholder->setShortcut(QKeySequence(Qt::ALT + Qt::Key_Tab));
+	if (actionPrevious_Completion_Placeholder->shortcut() == QKeySequence(Qt::CTRL + Qt::SHIFT + Qt::Key_Tab))
+		actionPrevious_Completion_Placeholder->setShortcut(QKeySequence(Qt::ALT + Qt::SHIFT + Qt::Key_Tab));
+#endif
 
-	connect(TWApp::instance(), SIGNAL(highlightLineOptionChanged()), this, SLOT(resetExtraSelections()));
-	
 	cursorPositionChangedSlot();
 	updateLineNumberAreaWidth(0);
 	updateColors();
+	TWUtils::installCustomShortcuts(this);
 }
 
 void CompletingEdit::prefixLines(const QString &prefix)
@@ -165,13 +185,18 @@ void CompletingEdit::unPrefixLines(const QString &prefix)
 
 void CompletingEdit::updateColors()
 {
-	Q_ASSERT(currentCompletionFormat != NULL);
-	Q_ASSERT(braceMatchingFormat != NULL);
-	Q_ASSERT(currentLineFormat != NULL);
-	Q_ASSERT(lineNumberArea != NULL);
+	Q_ASSERT(currentCompletionFormat);
+	Q_ASSERT(braceMatchingFormat);
+	Q_ASSERT(currentLineFormat);
+	Q_ASSERT(lineNumberArea);
 
-	qreal bgR, bgG, bgB;
-	qreal fgR, fgG, fgB;
+#if QT_VERSION < QT_VERSION_CHECK(6, 0, 0)
+	qreal bgR{1}, bgG{1}, bgB{1};
+	qreal fgR{0}, fgG{0}, fgB{0};
+#else
+	float bgR{1}, bgG{1}, bgB{1};
+	float fgR{0}, fgG{0}, fgB{0};
+#endif
 
 	palette().color(QPalette::Active, QPalette::Base).getRgbF(&bgR, &bgG, &bgB);
 	palette().color(QPalette::Active, QPalette::Text).getRgbF(&fgR, &fgG, &fgB);
@@ -189,7 +214,7 @@ void CompletingEdit::updateColors()
 
 CompletingEdit::~CompletingEdit()
 {
-	setCompleter(NULL);
+	setCompleter(nullptr);
 }
 
 void CompletingEdit::setCompleter(QCompleter *completer)
@@ -203,11 +228,9 @@ void CompletingEdit::setCompleter(QCompleter *completer)
 
 void CompletingEdit::cursorPositionChangedSlot()
 {
-	setCompleter(NULL);
-	if (!currentCompletionRange.isNull()) {
-		QTextCursor curs = textCursor();
+	setCompleter(nullptr);
+	if (!currentCompletionRange.isNull())
 		currentCompletionRange = QTextCursor();
-	}
 	resetExtraSelections();
 	prefixLength = 0;
 }
@@ -271,7 +294,7 @@ void CompletingEdit::mouseMoveEvent(QMouseEvent *e)
 		case none:
 			QTextEdit::mouseMoveEvent(e);
 			return;
-		
+
 		case synctexClick:
 		case ignoring:
 			e->accept();
@@ -280,7 +303,7 @@ void CompletingEdit::mouseMoveEvent(QMouseEvent *e)
 		case extendingSelection:
 			QTextEdit::mouseMoveEvent(e);
 			return;
-			
+
 		case normalSelection:
 			if (clickCount == 1
 				&& dragStartCursor.position() >= textCursor().selectionStart()
@@ -324,7 +347,7 @@ void CompletingEdit::mouseMoveEvent(QMouseEvent *e)
 								// Otherwise, simply remove the source text
 								source.removeSelectedText();
 							}
-							
+
 							if (!insideWindow) {
 								// The selection was moved to a different window,
 								// so dropCursor has no sensible data here. Thus,
@@ -343,12 +366,13 @@ void CompletingEdit::mouseMoveEvent(QMouseEvent *e)
 			}
 			setTextCursor(dragStartCursor);
 			mouseMode = dragSelecting;
-			// fall through to dragSelecting
+			// fall through
+			// to dragSelecting
 
 		case dragSelecting:
 			QPoint pos = e->pos();
 			int scrollValue = -1;
-			if (verticalScrollBar() != NULL) {
+			if (verticalScrollBar()) {
 				if (pos.y() < frameRect().top())
 					verticalScrollBar()->triggerAction(QScrollBar::SliderSingleStepSub);
 				else if (pos.y() > frameRect().bottom())
@@ -394,28 +418,29 @@ void CompletingEdit::mouseReleaseEvent(QMouseEvent *e)
 			QTextEdit::mouseReleaseEvent(e);
 			return;
 		case ignoring:
+			mouseMode = none;
 			e->accept();
 			return;
 		case synctexClick:
+			mouseMode = none;
 			{
 				QTextCursor curs = cursorForPosition(e->pos());
-#if QT_VERSION < 0x040700
-				emit syncClick(curs.blockNumber() + 1, curs.position() - curs.block().position());
-#else
 				emit syncClick(curs.blockNumber() + 1, curs.positionInBlock());
-#endif
 			}
 			e->accept();
 			return;
 		case dragSelecting:
+			mouseMode = none;
 			e->accept();
 			return;
 		case normalSelection:
+			mouseMode = none;
 			setTextCursor(dragStartCursor);
 			setSelectionClipboard(dragStartCursor);
 			e->accept();
 			return;
 		case extendingSelection:
+			mouseMode = none;
 			QTextEdit::mouseReleaseEvent(e);
 			return;
 	}
@@ -439,8 +464,8 @@ bool CompletingEdit::selectWord(QTextCursor& cursor)
 	if (text.length() < 1) // empty line
 		return false;
 
-	int start, end;
-	bool result = TWUtils::findNextWord(text, cursor.selectionStart() - block.position(), start, end);
+	int start{0}, end{0};
+	bool result = Tw::Document::TeXDocument::findNextWord(text, cursor.selectionStart() - block.position(), start, end);
 	cursor.setPosition(block.position() + start);
 	cursor.setPosition(block.position() + end, QTextCursor::KeepAnchor);
 
@@ -534,7 +559,7 @@ void CompletingEdit::timerEvent(QTimerEvent *e)
 	else
 		QTextEdit::timerEvent(e);
 }
-	
+
 void CompletingEdit::focusInEvent(QFocusEvent *e)
 {
 	if (c)
@@ -562,11 +587,12 @@ void CompletingEdit::resetExtraSelections()
 
 void CompletingEdit::keyPressEvent(QKeyEvent *e)
 {
-	// Shortcut key for command completion
-	bool isShortcut = (e->key() == Qt::Key_Tab || e->key() == Qt::Key_Backtab);
-	if (isShortcut && autocompleteEnabled) {
-		handleCompletionShortcut(e);
-		return;
+	if (autocompleteEnabled) {
+		QKeySequence seq(static_cast<int>(e->modifiers()) | e->key());
+		if (seq == actionNext_Completion->shortcut() || seq == actionPrevious_Completion->shortcut() || seq == actionNext_Completion_Placeholder->shortcut() || seq == actionPrevious_Completion_Placeholder->shortcut()) {
+			if (handleCompletionShortcut(e))
+				return;
+		}
 	}
 
 	if (!e->text().isEmpty())
@@ -577,6 +603,11 @@ void CompletingEdit::keyPressEvent(QKeyEvent *e)
 			handleReturn(e);
 			break;
 
+		case Qt::Key_Tab:
+		case Qt::Key_Backtab:
+			handleTab(e);
+			break;
+
 		case Qt::Key_Backspace:
 			handleBackspace(e);
 			break;
@@ -592,7 +623,7 @@ void CompletingEdit::handleReturn(QKeyEvent *e)
 	QString prefix;
 	// Check if auto indent is on and applicable
 	if (autoIndentMode >= 0 && autoIndentMode < indentModes->count() && e->modifiers() == Qt::NoModifier) {
-		QRegExp &re = (*indentModes)[autoIndentMode].regex;
+		const QRegularExpression &re = (*indentModes)[autoIndentMode].regex;
 		// Only apply prefix recognition to characters in front of the cursor.
 		// Otherwise, we would accumulate characters if the cursor is inside the
 		// region matched by the regexp.
@@ -606,8 +637,9 @@ void CompletingEdit::handleReturn(QKeyEvent *e)
 		curs.movePosition(QTextCursor::StartOfBlock, QTextCursor::KeepAnchor);
 		QString blockText = curs.selectedText();
 		// Check if the prefix matches the regexp of the current auto indent mode
-		if (blockText.indexOf(re) == 0 && re.matchedLength() > 0)
-			prefix = blockText.left(re.matchedLength());
+		QRegularExpressionMatch m = re.match(blockText);
+		if (m.capturedStart() == 0 && m.capturedLength() > 0)
+			prefix = m.captured();
 	}
 	// Propagate the key press event to the base class so that the text is
 	// actually modified
@@ -641,11 +673,10 @@ void CompletingEdit::handleBackspace(QKeyEvent *e)
 
 void CompletingEdit::handleOtherKey(QKeyEvent *e)
 {
-	QTextCursor cursor = textCursor();
 	int pos = textCursor().selectionStart(); // remember cursor before the keystroke
 	int end = textCursor().selectionEnd();
 	QTextEdit::keyPressEvent(e);
-	cursor = textCursor();
+	QTextCursor cursor = textCursor();
 	bool arrowKey = false;
 	if (e->key() == Qt::Key_Left || e->key() == Qt::Key_Right) {
 		arrowKey = true;
@@ -682,11 +713,15 @@ void CompletingEdit::handleOtherKey(QKeyEvent *e)
 					sel.format = *braceMatchingFormat;
 					selList.append(sel);
 					setExtraSelections(selList);
+#if QT_VERSION < QT_VERSION_CHECK(5, 4, 0)
 					QTimer::singleShot(250, this, SLOT(resetExtraSelections()));
+#else
+					QTimer::singleShot(250, this, &CompletingEdit::resetExtraSelections);
+#endif
 				}
 			}
 		}
-	}	
+	}
 }
 
 void CompletingEdit::setSmartQuotesMode(int index)
@@ -697,7 +732,7 @@ void CompletingEdit::setSmartQuotesMode(int index)
 QStringList CompletingEdit::smartQuotesModes()
 {
 	loadSmartQuotesModes();
-	
+
 	QStringList modes;
 	foreach (const QuotesMode& mode, *quotesModes)
 		modes << mode.name;
@@ -706,33 +741,35 @@ QStringList CompletingEdit::smartQuotesModes()
 
 void CompletingEdit::loadSmartQuotesModes()
 {
-	if (quotesModes == NULL) {
-		QDir configDir(TWUtils::getLibraryPath(QString::fromLatin1("configuration")));
+	if (!quotesModes) {
+		QDir configDir(Tw::Utils::ResourcesLibrary::getLibraryPath(QStringLiteral("configuration")));
 		quotesModes = new QList<QuotesMode>;
 		QFile quotesModesFile(configDir.filePath(QString::fromLatin1("smart-quotes-modes.txt")));
 		if (quotesModesFile.open(QIODevice::ReadOnly)) {
-			QRegExp modeName(QString::fromLatin1("\\[([^]]+)\\]"));
-			QRegExp quoteLine(QString::fromLatin1("([^ \\t])\\s+([^ \\t]+)\\s+([^ \\t]+)"));
+			QRegularExpression modeName(QStringLiteral("^\\[([^]]+)\\]$"));
+			QRegularExpression quoteLine(QStringLiteral("^([^ \\t])\\s+([^ \\t]+)\\s+([^ \\t]+)$"));
 			QuotesMode newMode;
-			while (1) {
+			while (true) {
 				QByteArray ba = quotesModesFile.readLine();
 				if (ba.size() == 0)
 					break;
 				if (ba[0] == '#' || ba[0] == '\n')
 					continue;
 				QString line = QString::fromUtf8(ba.data(), ba.size()).trimmed();
-				if (modeName.exactMatch(line)) {
+				QRegularExpressionMatch modeNameMatch = modeName.match(line);
+				if (modeNameMatch.hasMatch()) {
 					if (newMode.mappings.count() > 0) {
 						quotesModes->append(newMode);
 						newMode.mappings.clear();
 					}
-					newMode.name = modeName.cap(1);
+					newMode.name = modeNameMatch.captured(1);
 					continue;
 				}
-				if (quoteLine.exactMatch(line) && newMode.name.length() > 0) {
-					QChar key = quoteLine.cap(1)[0];
-					const QString& open = quoteLine.cap(2);
-					const QString& close = quoteLine.cap(3);
+				QRegularExpressionMatch quoteLineMatch = quoteLine.match(line);
+				if (quoteLineMatch.hasMatch() && newMode.name.length() > 0) {
+					QChar key = quoteLineMatch.captured(1)[0];
+					const QString& open = quoteLineMatch.captured(2);
+					const QString& close = quoteLineMatch.captured(3);
 					newMode.mappings[key] = QuotePair(open,close);
 					continue;
 				}
@@ -762,7 +799,7 @@ void CompletingEdit::maybeSmartenQuote(int offset)
 	QuoteMapping::const_iterator iter = mappings.find(keyChar);
 	if (iter == mappings.end())
 		return;
-	
+
 	replacement = iter.value().second;
 	if (offset == 0) {
 		// always use opening quotes at the beginning of the document
@@ -771,12 +808,12 @@ void CompletingEdit::maybeSmartenQuote(int offset)
 	else {
 		if (text[offset - 1].isSpace())
 			replacement = iter.value().first;
-		
+
 		// after opening brackets, also use opening quotes
 		if (text[offset - 1] == QChar::fromLatin1('{') || text[offset - 1] == QChar::fromLatin1('[') || text[offset - 1] == QChar::fromLatin1('('))
 			replacement = iter.value().first;
 	}
-	
+
 	cursor.insertText(replacement);
 }
 
@@ -818,24 +855,16 @@ void CompletingEdit::smartenQuotes()
 	}
 }
 
-void CompletingEdit::handleCompletionShortcut(QKeyEvent *e)
+// \returns true if shortcut was handled, false otherwise
+bool CompletingEdit::handleCompletionShortcut(QKeyEvent *e)
 {
-// usage:
-//   unmodified: next completion
-//   shift     : previous completion
-//   ctl/alt       : skip to next placeholder (alt on Mac, ctl elsewhere)
-//   ctl/alt-shift : skip to previous placeholder
-
-#if defined(Q_OS_DARWIN)
-	if ((e->modifiers() & ~Qt::ShiftModifier) == Qt::AltModifier)
-#else
-	if ((e->modifiers() & ~Qt::ShiftModifier) == Qt::ControlModifier)
-#endif
+	QKeySequence seq(static_cast<int>(e->modifiers()) | e->key());
+	if (seq == actionNext_Completion_Placeholder->shortcut() || seq == actionPrevious_Completion_Placeholder->shortcut())
 	{
-		if (!find(QString(0x2022), (e->modifiers() & Qt::ShiftModifier)
-									? QTextDocument::FindBackward : (QTextDocument::FindFlags)0))
+		if (!find(QString(QChar(0x2022)), (seq == actionPrevious_Completion_Placeholder->shortcut())
+									? QTextDocument::FindBackward : QTextDocument::FindFlags()))
 			QApplication::beep();
-		return;
+		return true;
 	}
 
 	// if we are at the beginning of the line (i.e., only whitespaces before a
@@ -850,8 +879,8 @@ void CompletingEdit::handleCompletionShortcut(QKeyEvent *e)
 		if(lineStartCursor.selectedText().trimmed().isEmpty())
 			atLineStart = true;
 	}
-	
-	if (c == NULL && !atLineStart) {
+
+	if (!c && !atLineStart) {
 		cmpCursor = textCursor();
 		if (!selectWord(cmpCursor) && textCursor().selectionStart() > 0) {
 			cmpCursor.setPosition(textCursor().selectionStart() - 1);
@@ -884,8 +913,8 @@ void CompletingEdit::handleCompletionShortcut(QKeyEvent *e)
 				}
 			}
 		}
-		
-		while (1) {
+
+		while (true) {
 			QString completionPrefix = cmpCursor.selectedText();
 			if (!completionPrefix.isEmpty()) {
 				setCompleter(sharedCompleter);
@@ -897,24 +926,24 @@ void CompletingEdit::handleCompletionShortcut(QKeyEvent *e)
 						cmpCursor.setPosition(end, QTextCursor::KeepAnchor);
 						continue;
 					}
-					setCompleter(NULL);
+					setCompleter(nullptr);
 				}
 				else {
-					if (e->modifiers() == Qt::ShiftModifier)
+					if (seq == actionPrevious_Completion->shortcut())
 						c->setCurrentRow(c->completionCount() - 1);
 					showCurrentCompletion();
-					return;
+					return true;
 				}
 			}
 			break;
 		}
 	}
-	
-	if (c != NULL && c->completionCount() > 0) {
-		if (e->modifiers() == Qt::ShiftModifier)  {
+
+	if (c && c->completionCount() > 0) {
+		if (seq == actionPrevious_Completion->shortcut()) {
 			if (c->currentRow() == 0) {
 				showCompletion(c->completionPrefix());
-				setCompleter(NULL);
+				setCompleter(nullptr);
 			}
 			else {
 				c->setCurrentRow(c->currentRow() - 1);
@@ -924,17 +953,21 @@ void CompletingEdit::handleCompletionShortcut(QKeyEvent *e)
 		else {
 			if (c->currentRow() == c->completionCount() - 1) {
 				showCompletion(c->completionPrefix());
-				setCompleter(NULL);
+				setCompleter(nullptr);
 			}
 			else {
 				c->setCurrentRow(c->currentRow() + 1);
 				showCurrentCompletion();
 			}
 		}
-		return;
+		return true;
 	}
-	
-	if(!noSelection) {
+	return false;
+}
+
+void CompletingEdit::handleTab(QKeyEvent * e)
+{
+	if (textCursor().hasSelection()) {
 		if(e->modifiers() == Qt::ShiftModifier) {
 			unPrefixLines(QString::fromLatin1("\t"));
 		} else {
@@ -947,7 +980,7 @@ void CompletingEdit::handleCompletionShortcut(QKeyEvent *e)
 
 void CompletingEdit::showCompletion(const QString& completion, int insOffset)
 {
-	disconnect(this, SIGNAL(cursorPositionChanged()), this, SLOT(cursorPositionChangedSlot()));
+	disconnect(this, &CompletingEdit::cursorPositionChanged, this, &CompletingEdit::cursorPositionChangedSlot);
 
 	if (c->widget() != this)
 		return;
@@ -969,7 +1002,7 @@ void CompletingEdit::showCompletion(const QString& completion, int insOffset)
 	currentCompletionRange = cmpCursor;
 	resetExtraSelections();
 
-	connect(this, SIGNAL(cursorPositionChanged()), this, SLOT(cursorPositionChangedSlot()));
+	connect(this, &CompletingEdit::cursorPositionChanged, this, &CompletingEdit::cursorPositionChangedSlot);
 }
 
 void CompletingEdit::showCurrentCompletion()
@@ -1002,7 +1035,7 @@ void CompletingEdit::showCurrentCompletion()
 	}
 
 	QString completion = model->item(items[itemIndex]->row(), 1)->text();
-	
+
 	int insOffset = completion.indexOf(QLatin1String("#INS#"));
 	if (insOffset != -1)
 		completion.replace(QLatin1String("#INS#"), QLatin1String(""));
@@ -1015,10 +1048,12 @@ void CompletingEdit::loadCompletionsFromFile(QStandardItemModel *model, const QS
 	QFile	completionFile(filename);
 	if (completionFile.exists() && completionFile.open(QIODevice::ReadOnly | QIODevice::Text)) {
 		QTextStream in(&completionFile);
+#if QT_VERSION < QT_VERSION_CHECK(6, 0, 0)
 		in.setCodec("UTF-8");
+#endif
 		in.setAutoDetectUnicode(true);
 		QList<QStandardItem*> row;
-		while (1) {
+		while (true) {
 			QString	line = in.readLine();
 			if (line.isNull())
 				break;
@@ -1044,7 +1079,7 @@ void CompletingEdit::loadCompletionFiles(QCompleter *theCompleter)
 {
 	QStandardItemModel *model = new QStandardItemModel(0, 2, theCompleter); // columns are abbrev, expansion
 
-	QDir completionDir(TWUtils::getLibraryPath(QString::fromLatin1("completion")));
+	QDir completionDir(Tw::Utils::ResourcesLibrary::getLibraryPath(QStringLiteral("completion")));
 	foreach (QFileInfo fileInfo, completionDir.entryInfoList(QDir::Files | QDir::Readable, QDir::Name)) {
 		loadCompletionsFromFile(model, fileInfo.canonicalFilePath());
 	}
@@ -1052,10 +1087,17 @@ void CompletingEdit::loadCompletionFiles(QCompleter *theCompleter)
 	theCompleter->setModel(model);
 }
 
-void CompletingEdit::jumpToPdf()
+void CompletingEdit::jumpToPdf(QTextCursor pos)
+{
+	if (pos.isNull())
+		pos = textCursor();
+	emit syncClick(pos.blockNumber() + 1, pos.positionInBlock());
+}
+
+void CompletingEdit::jumpToPdfFromContextMenu()
 {
 	QAction *act = qobject_cast<QAction*>(sender());
-	if (act != NULL) {
+	if (act) {
 		QPoint pt = act->data().toPoint();
 		emit syncClick(pt.y(), pt.x());
 	}
@@ -1064,67 +1106,57 @@ void CompletingEdit::jumpToPdf()
 void CompletingEdit::contextMenuEvent(QContextMenuEvent *event)
 {
 	QMenu *menu = createStandardContextMenu();
-	QAction *defaultAction = NULL;
+	QAction *defaultAction = nullptr;
 	QAction *act = new QAction(tr("Jump to PDF"), menu);
 	QTextCursor cur = cursorForPosition(event->pos());
 
-#if QT_VERSION < 0x040700
-	act->setData(QVariant(QPoint(cur.position() - cur.block().position(), cur.blockNumber() + 1)));
-#else
 	act->setData(QVariant(QPoint(cur.positionInBlock(), cur.blockNumber() + 1)));
-#endif
-	connect(act, SIGNAL(triggered()), this, SLOT(jumpToPdf()));
+	connect(act, &QAction::triggered, this, &CompletingEdit::jumpToPdfFromContextMenu);
 	menu->insertSeparator(menu->actions().first());
 	menu->insertAction(menu->actions().first(), act);
-	
-	if (pHunspell != NULL) {
+
+	const Tw::Document::SpellChecker::Dictionary * dictionary = getSpellChecker();
+	if (dictionary) {
 		currentWord = cursorForPosition(event->pos());
 		currentWord.setPosition(currentWord.position());
 		if (selectWord(currentWord)) {
-			QByteArray word = spellingCodec->fromUnicode(currentWord.selectedText());
-			int spellResult = Hunspell_spell(pHunspell, word.data());
-			if (spellResult == 0) {
-				char **suggestionList;
-				int count = Hunspell_suggest(pHunspell, &suggestionList, word.data());
+			if (!dictionary->isWordCorrect(currentWord.selectedText())) {
 				QAction *sep = menu->insertSeparator(menu->actions().first());
-				if (count == 0)
+
+				QList<QString> suggestions = dictionary->suggestionsForWord(currentWord.selectedText());
+				if (suggestions.size() == 0)
 					menu->insertAction(sep, new QAction(tr("No suggestions"), menu));
 				else {
 					QSignalMapper *mapper = new QSignalMapper(menu);
-					for (int i = 0; i < count; ++i) {
-						QString str = spellingCodec->toUnicode(suggestionList[i]);
+					foreach(const QString & str, suggestions) {
 						act = new QAction(str, menu);
-						connect(act, SIGNAL(triggered()), mapper, SLOT(map()));
+						connect(act, &QAction::triggered, mapper, static_cast<void (QSignalMapper::*)()>(&QSignalMapper::map));
 						mapper->setMapping(act, str);
 						menu->insertAction(sep, act);
-						free(suggestionList[i]);
 						if (!defaultAction)
 							defaultAction = act;
 					}
-					free(suggestionList);
-					connect(mapper, SIGNAL(mapped(const QString&)), this, SLOT(correction(const QString&)));
+#if QT_VERSION < QT_VERSION_CHECK(5, 15, 0)
+					connect(mapper, static_cast<void (QSignalMapper::*)(const QString &)>(&QSignalMapper::mapped), this, &CompletingEdit::correction);
+#else
+					connect(mapper, &QSignalMapper::mappedString, this, &CompletingEdit::correction);
+#endif
 				}
 				sep = menu->insertSeparator(menu->actions().first());
 //				QAction *add = new QAction(tr("Add to dictionary"), menu);
-//				connect(add, SIGNAL(triggered()), this, SLOT(addToDictionary()));
+//				connect(add, &QAction::triggered, this, &CompletingEdit::addToDictionary);
 //				menu->insertAction(sep, add);
 				QAction *ignore = new QAction(tr("Ignore word"), menu);
-				connect(ignore, SIGNAL(triggered()), this, SLOT(ignoreWord()));
+				connect(ignore, &QAction::triggered, this, &CompletingEdit::ignoreWord);
 				menu->insertAction(sep, ignore);
 			}
 		}
 	}
-	
+
 	menu->exec(event->globalPos(), defaultAction);
 	delete menu;
 }
 
-void CompletingEdit::setSpellChecker(Hunhandle* h, QTextCodec *codec)
-{
-	pHunspell = h;
-	spellingCodec = codec;
-}
-
 void CompletingEdit::setAutoIndentMode(int index)
 {
 	autoIndentMode = (index >= 0 && index < indentModes->count()) ? index : -1;
@@ -1143,31 +1175,34 @@ void CompletingEdit::addToDictionary()
 
 void CompletingEdit::ignoreWord()
 {
+	Tw::Document::SpellChecker::Dictionary * dictionary = getSpellChecker();
+	if (dictionary == nullptr)
+		return;
 	// note that this is not persistent after quitting TW
-	QByteArray word = spellingCodec->fromUnicode(currentWord.selectedText());
-	(void)Hunspell_add(pHunspell, word.data());
+	dictionary->ignoreWord(currentWord.selectedText());
 	emit rehighlight();
 }
 
 void CompletingEdit::loadIndentModes()
 {
-	if (indentModes == NULL) {
-		QDir configDir(TWUtils::getLibraryPath(QString::fromLatin1("configuration")));
+	if (!indentModes) {
+		QDir configDir(Tw::Utils::ResourcesLibrary::getLibraryPath(QStringLiteral("configuration")));
 		indentModes = new QList<IndentMode>;
 		QFile indentPatternFile(configDir.filePath(QString::fromLatin1("auto-indent-patterns.txt")));
 		if (indentPatternFile.open(QIODevice::ReadOnly)) {
-			QRegExp re(QString::fromLatin1("\"([^\"]+)\"\\s+(.+)"));
-			while (1) {
+			QRegularExpression re(QStringLiteral("^\"([^\"]+)\"\\s+(.+)$"));
+			while (true) {
 				QByteArray ba = indentPatternFile.readLine();
 				if (ba.size() == 0)
 					break;
 				if (ba[0] == '#' || ba[0] == '\n')
 					continue;
 				QString line = QString::fromUtf8(ba.data(), ba.size()).trimmed();
-				if (re.exactMatch(line)) {
+				QRegularExpressionMatch m = re.match(line);
+				if (m.hasMatch()) {
 					IndentMode mode;
-					mode.name = re.cap(1);
-					mode.regex = QRegExp(re.cap(2).trimmed());
+					mode.name = m.captured(1);
+					mode.regex = QRegularExpression(m.captured(2).trimmed());
 					if (!mode.name.isEmpty() && mode.regex.isValid())
 						indentModes->append(mode);
 				}
@@ -1179,7 +1214,7 @@ void CompletingEdit::loadIndentModes()
 QStringList CompletingEdit::autoIndentModes()
 {
 	loadIndentModes();
-	
+
 	QStringList modes;
 	foreach (const IndentMode& mode, *indentModes)
 		modes << mode.name;
@@ -1196,7 +1231,11 @@ void CompletingEdit::dragEnterEvent(QDragEnterEvent *event)
 
 void CompletingEdit::dropEvent(QDropEvent *event)
 {
+#if QT_VERSION < QT_VERSION_CHECK(6, 0, 0)
 	QTextCursor dropCursor = cursorForPosition(event->pos());
+#else
+  QTextCursor dropCursor = cursorForPosition(event->position().toPoint());
+#endif
 	if (!dropCursor.isNull()) {
 		droppedOffset = dropCursor.position();
 		droppedLength = event->mimeData()->text().length();
@@ -1233,20 +1272,6 @@ void CompletingEdit::setLineNumberDisplay(bool displayNumbers)
 	updateLineNumberAreaWidth(0);
 }
 
-int CompletingEdit::lineNumberAreaWidth()
-{
-	int digits = 1;
-	int max = qMax(1, document()->blockCount());
-	while (max >= 10) {
-		max /= 10;
-		++digits;
-	}
-	
-	int space = 3 + fontMetrics().width(QChar::fromLatin1('9')) * digits;
-	
-	return space;
-}
-
 bool CompletingEdit::getLineNumbersVisible() const
 {
 	return lineNumberArea->isVisible();
@@ -1255,7 +1280,7 @@ bool CompletingEdit::getLineNumbersVisible() const
 void CompletingEdit::updateLineNumberAreaWidth(int /* newBlockCount */)
 {
 	if (lineNumberArea->isVisible()) {
-		setViewportMargins(lineNumberAreaWidth(), 0, 0, 0);
+		setViewportMargins(lineNumberArea->sizeHint().width(), 0, 0, 0);
 		lineNumberArea->update();
 	}
 	else {
@@ -1269,7 +1294,7 @@ void CompletingEdit::updateLineNumberArea(const QRect &rect, int dy)
 		lineNumberArea->scroll(0, dy);
 	else
 		lineNumberArea->update(0, rect.y(), lineNumberArea->width(), rect.height());
-	
+
 	if (rect.contains(viewport()->rect()))
 		updateLineNumberAreaWidth(0);
 }
@@ -1277,16 +1302,16 @@ void CompletingEdit::updateLineNumberArea(const QRect &rect, int dy)
 void CompletingEdit::resizeEvent(QResizeEvent *e)
 {
 	QTextEdit::resizeEvent(e);
-	
+
 	QRect cr = contentsRect();
-	lineNumberArea->setGeometry(QRect(cr.left(), cr.top(), lineNumberAreaWidth(), cr.height()));
+	lineNumberArea->setGeometry(QRect(cr.left(), cr.top(), lineNumberArea->sizeHint().width(), cr.height()));
 }
 
 void CompletingEdit::wheelEvent(QWheelEvent *e)
 {
 	if (e->modifiers() & Qt::ControlModifier)
 	{
-		wheelDelta += e->delta();  // accumulate wheelDelta for high-resolution mice, which might pass small values.
+		wheelDelta += e->angleDelta().y();  // accumulate wheelDelta for high-resolution mice, which might pass small values.
 		int sign = (wheelDelta < 0) ? -1 : 1;
 		const int stepSize = 120;  // according to Qt docs a standard wheel step corresponds to a delta of 120.
 		int steps = (sign * wheelDelta) / stepSize;  // abs value to guarantee rounding towards 0.
@@ -1304,39 +1329,9 @@ void CompletingEdit::wheelEvent(QWheelEvent *e)
 	QTextEdit::wheelEvent(e);
 }
 
-void CompletingEdit::lineNumberAreaPaintEvent(QPaintEvent *event)
-{
-	Q_ASSERT(lineNumberArea != NULL);
-
-	QPainter painter(lineNumberArea);
-	painter.fillRect(event->rect(), lineNumberArea->bgColor());
-	
-	QTextBlock block = document()->begin();
-	int blockNumber = 1;
-
-	QAbstractTextDocumentLayout *layout = document()->documentLayout();
-	int top = layout->blockBoundingRect(block).top() - verticalScrollBar()->value();
-	int bottom = top + layout->blockBoundingRect(block).height();
-	
-	while (block.isValid() && top <= event->rect().bottom()) {
-		if (bottom >= event->rect().top()) {
-			QString number = QString::number(blockNumber);
-			painter.drawText(0, top, lineNumberArea->width() - 1, fontMetrics().height(),
-							 Qt::AlignRight, number);
-		}
-
-		block = block.next();
-		if (block == document()->end())
-			break;
-		top = bottom;
-		bottom = top + (int)layout->blockBoundingRect(block).height();
-		++blockNumber;
-	}
-}
-
 void CompletingEdit::setTextCursor(const QTextCursor & cursor)
 {
-	// QTextEdit::setTextCursor only scrolls to cursor.position(). If 
+	// QTextEdit::setTextCursor only scrolls to cursor.position(). If
 	// position() > anchor(), the two are on different lines, and the view has
 	// to scroll up, this means that not the whole selection is visible.
 	// By manually setting the cursor to anchor() first, we ensure that the
@@ -1349,6 +1344,16 @@ void CompletingEdit::setTextCursor(const QTextCursor & cursor)
 	QTextEdit::setTextCursor(cursor);
 }
 
+void CompletingEdit::setDocument(QTextDocument * document)
+{
+	disconnect(QTextEdit::document(), nullptr, this, nullptr);
+	// Remember the cursor width setting
+	int oldCursorWidth = cursorWidth();
+	QTextEdit::setDocument(document);
+	setCursorWidth(oldCursorWidth);
+	connect(document, &QTextDocument::blockCountChanged, this, &CompletingEdit::updateLineNumberAreaWidth);
+}
+
 bool CompletingEdit::event(QEvent *e)
 {
 	if (e->type() == QEvent::UpdateRequest) {
@@ -1360,6 +1365,22 @@ bool CompletingEdit::event(QEvent *e)
 	// derive the colors from the application's palette
 	if (e->type() == QEvent::PaletteChange)
 		updateColors();
+	if (e->type() == QEvent::ShortcutOverride) {
+		auto ke = reinterpret_cast<QKeyEvent*>(e);
+		QKeySequence seq(static_cast<int>(ke->modifiers()) | ke->key());
+		if (seq == actionNext_Completion->shortcut() ||
+			seq == actionPrevious_Completion->shortcut() ||
+			seq == actionNext_Completion_Placeholder->shortcut() ||
+			seq == actionPrevious_Completion_Placeholder->shortcut())
+		{
+			// If the key press corresponds to a completion shortcut, accept the
+			// event to tell Qt not to treat it as a shortcut but to send it to
+			// the focused widget normally (thereby invoking the keyPressEvent
+			// handler)
+			e->accept();
+		}
+	}
+
 	return QTextEdit::event(e);
 }
 
@@ -1371,6 +1392,17 @@ void CompletingEdit::scrollContentsBy(int dx, int dy)
 	QTextEdit::scrollContentsBy(dx, dy);
 }
 
+Tw::Document::SpellChecker::Dictionary * CompletingEdit::getSpellChecker() const
+{
+	Tw::Document::TeXDocument * doc = qobject_cast<Tw::Document::TeXDocument *>(document());
+	if (doc == nullptr)
+		return nullptr;
+	TeXHighlighter * highlighter = doc->getHighlighter();
+	if (highlighter == nullptr)
+		return nullptr;
+	return highlighter->getSpellChecker();
+}
+
 void CompletingEdit::setHighlightCurrentLine(bool highlight)
 {
 	if (highlight != highlightCurrentLine) {
@@ -1417,13 +1449,13 @@ void CompletingEdit::setFontWeight(int weight)
 }
 
 
-QTextCharFormat	*CompletingEdit::currentCompletionFormat = NULL;
-QTextCharFormat	*CompletingEdit::braceMatchingFormat = NULL;
-QTextCharFormat	*CompletingEdit::currentLineFormat = NULL;
+QTextCharFormat	*CompletingEdit::currentCompletionFormat = nullptr;
+QTextCharFormat	*CompletingEdit::braceMatchingFormat = nullptr;
+QTextCharFormat	*CompletingEdit::currentLineFormat = nullptr;
 bool CompletingEdit::highlightCurrentLine = true;
 bool CompletingEdit::autocompleteEnabled = true;
 
-QCompleter	*CompletingEdit::sharedCompleter = NULL;
+QCompleter	*CompletingEdit::sharedCompleter = nullptr;
 
-QList<CompletingEdit::IndentMode> *CompletingEdit::indentModes = NULL;
-QList<CompletingEdit::QuotesMode> *CompletingEdit::quotesModes = NULL;
+QList<CompletingEdit::IndentMode> *CompletingEdit::indentModes = nullptr;
+QList<CompletingEdit::QuotesMode> *CompletingEdit::quotesModes = nullptr;
