@@ -1,6 +1,6 @@
 /*
 	This is part of TeXworks, an environment for working with TeX documents
-	Copyright (C) 2008-2016  Jonathan Kew, Stefan Löffler, Charlie Sharpsteen
+	Copyright (C) 2008-2020  Jonathan Kew, Stefan Löffler, Charlie Sharpsteen
 
 	This program is free software; you can redistribute it and/or modify
 	it under the terms of the GNU General Public License as published by
@@ -22,20 +22,19 @@
 #ifndef ConfirmDelete_H
 #define ConfirmDelete_H
 
-#include <QDialog>
-#include <QStringList>
-
-#include "TWUtils.h"
-
 #include "ui_ConfirmDelete.h"
 
+#include <QDialog>
+#include <QDir>
+#include <QStringList>
+
 class ConfirmDelete : public QDialog, private Ui::ConfirmDelete
 {
 	Q_OBJECT
 
 public:
-	ConfirmDelete(QWidget *parent = NULL);
-	virtual ~ConfirmDelete();
+	ConfirmDelete(QWidget *parent = nullptr);
+	~ConfirmDelete() override = default;
 
 	static void doConfirmDelete(const QDir& dir, const QStringList& fileList);
 
